@@ -23,7 +23,11 @@
 
 #--------------------------------------------------------------------------------------
 """
-This script is responsible for cluster environment setup for the given Amazon EC2 reservation
+Configures the acquired ''cluster'' to share the same folder across all the ''cluster''
+nodes (uses NFS protocol for this purpose). The cluster can be obtained through 
+its identifier as result of
+: ''a_cluster_id=`[[cloudflu-reservation-run]]` && echo ${a_cluster_id}'' 
+command or extracted from output of ''[[cloudflu-reservation-ls]]'' utility.
 """
 
 
@@ -67,7 +71,7 @@ def main() :
    
     AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.security_options.extract( an_option_parser )
     
-    a_reservation_id = ec2.use.options.extract( an_option_parser )
+    a_cluster_id = ec2.use.options.extract( an_option_parser )
    
     from cloudflu.preferences import get
     a_cluster_location = get( 'amazon.cluster.location' )
@@ -78,7 +82,7 @@ def main() :
     print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
     import sys; an_engine = sys.argv[ 0 ]
    
-    a_call = "%s %s" % ( an_engine, ec2.use.options.compose( a_reservation_id ) )
+    a_call = "%s %s" % ( an_engine, ec2.use.options.compose( a_cluster_id ) )
     print_d( a_call + '\n' )
    
 
@@ -87,7 +91,7 @@ def main() :
    
     an_ec2_conn = ec2.common.region_connect( a_cluster_location, AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )
 
-    a_reservation = ec2.use.get_reservation( an_ec2_conn, a_reservation_id )
+    a_reservation = ec2.use.get_reservation( an_ec2_conn, a_cluster_id )
     print_d( '< %r > : %s\n' % ( a_reservation, a_reservation.instances ) )
    
     a_security_group = ec2.use.get_security_group( an_ec2_conn, a_reservation )
@@ -119,7 +123,7 @@ def main() :
     
     
     print_d( "\n------------------ Printing succussive pipeline arguments -----------------\n" )
-    ec2.use.options.track( a_reservation_id )
+    ec2.use.options.track( a_cluster_id )
     
     
     print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
