@@ -23,7 +23,10 @@
 
 #--------------------------------------------------------------------------------------
 """
-This script is responsible for cluster environment setup for the given Amazon EC2 reservation
+Configures the acquired ''cluster'' to run OpenMPI based functionality. 
+The cluster can be obtained through its identifier as result of
+: ''a_cluster_id=`[[cloudflu-reservation-run]]` && echo ${a_cluster_id}'' 
+command or extracted from output of ''[[cloudflu-reservation-ls]]'' utility.
 """
 
 
@@ -72,7 +75,7 @@ def main() :
 
     AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.security_options.extract( an_option_parser )
 
-    a_reservation_id = ec2.use.options.extract( an_option_parser )
+    a_cluster_id = ec2.use.options.extract( an_option_parser )
 
     a_hostfile = openmpi_config_options.extract( an_option_parser )
     
@@ -86,7 +89,7 @@ def main() :
     import sys; an_engine = sys.argv[ 0 ]
     
     a_call = "%s %s %s" % ( an_engine,
-                            ec2.use.options.compose( a_reservation_id ), 
+                            ec2.use.options.compose( a_cluster_id ), 
                             openmpi_config_options.compose( a_hostfile ) )
     print_d( a_call + '\n' )
 
@@ -97,7 +100,7 @@ def main() :
     an_ec2_conn = ec2.common.region_connect( a_cluster_location, AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )
 
     try:
-        a_reservation = ec2.use.get_reservation( an_ec2_conn, a_reservation_id )
+        a_reservation = ec2.use.get_reservation( an_ec2_conn, a_cluster_id )
         print_d( '< %r > : %s\n' % ( a_reservation, a_reservation.instances ) )
     except Exception, exc:
         print_e( '%s\n' % exc )
@@ -162,7 +165,7 @@ def main() :
     
     
     print_d( "\n------------------ Printing succussive pipeline arguments -----------------\n" )
-    ec2.use.options.track( a_reservation_id )
+    ec2.use.options.track( a_cluster_id )
     
 
     print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
