@@ -23,7 +23,10 @@
 
 #--------------------------------------------------------------------------------------
 """
-This script is responsible for the task packaging and sending it for execution in a cloud
+A starting point for ''[[cloudflu-cluster-start]]'' macro utility. 
+Acquires a new ''reservation'' which, after some predefined configuration steps, 
+will become a ''cluster''. Returns an identifier as its output which will be treated as
+corresponding ''cluster'' identifier afterwards.
 """
 
 
