@@ -23,7 +23,10 @@
 
 #--------------------------------------------------------------------------------------
 """
-This script is responsible for cluster environment setup for the given Amazon EC2 reservation
+Process the solver output that was launched by ''[[cloudflu-solver-start]]'' utility. 
+These two utility was specially designed not just run together, but do their work 
+in parallel. The common entry point for both of these utilities is ''study'', 
+which could be reserver in advance by calling ''[[cloudflu-study-book]]'' utility.
 """
 
 
@@ -63,6 +66,8 @@ def main() :
 
     common.concurrency_options.add( an_option_parser )
 
+    common.communication_options.add( an_option_parser )
+    
     amazon.security_options.add( an_option_parser )
     
     common.options.add( an_option_parser )
@@ -75,6 +80,8 @@ def main() :
     
     AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.security_options.extract( an_option_parser )
     
+    a_socket_timeout = common.communication_options.extract( an_option_parser )
+
     a_number_threads = common.concurrency_options.extract( an_option_parser )
 
     a_study_name = data_transfer_options.extract( an_option_parser )
@@ -161,21 +168,18 @@ def main() :
     
     import study_book; an_output_study = study_book.entry_point( a_root_object, a_study_name, True )
 
-    def Download( the_study_object, the_output_dir, the_number_threads ) :
-        if a_before_hook != None :
-            a_before_hook( the_study_object.name(), the_output_dir )
-            pass
-
-        import download; download.entry_point( the_study_object, the_output_dir, None, the_number_threads, True, True, True, a_time_hook )
-
-        if an_after_hook != None :
-            an_after_hook( the_output_dir )
-            pass
-
+    if a_before_hook != None :
+        a_before_hook( an_output_study.name(), the_output_dir )
         pass
-
-    Download( an_output_study, an_output_dir, a_number_threads )
     
+    import download; download.entry_point( an_output_study, an_output_dir, a_number_threads, a_socket_timeout, 
+                                           None, True, True, True, a_time_hook )
+    
+    if an_after_hook != None :
+        an_after_hook( the_output_dir )
+        pass
+    
+
     print_d( "a_spent_time = %s, sec\n" % a_spent_time )
     
     
