@@ -23,7 +23,11 @@
 
 #------------------------------------------------------------------------------------------
 """
-This script is responsible for efficient uploading of multi file data
+Seals ''study'' with the given name. 
+The ''sealing'' notion is used in context of ''[[cloudflu-download]]'' utility called
+with ''--wait'' option. ''Seal'' tells to the ''[[cloudflu-download]]'' that everything 
+is uploaded and it cloud start to restore the entire entity from corresponding parts 
+(in case of ''study'' it means, that all the files are uploaded).
 """
 
 
@@ -62,10 +66,6 @@ def main() :
 
     amazon.security_options.add( an_option_parser )
     
-    common.concurrency_options.add( an_option_parser )
-
-    common.communication_options.add( an_option_parser )
-
     common.options.add( an_option_parser )
 
     
@@ -74,10 +74,6 @@ def main() :
 
     common.options.extract( an_option_parser )
     
-    common.communication_options.extract( an_option_parser )
-
-    a_number_threads = common.concurrency_options.extract( an_option_parser )
-    
     AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.security_options.extract( an_option_parser )
     
     from study_seal_options import extract as extract_options
